// Copyright 2010 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package cmplx64

import "github.com/AE9RB/math32"

// IsNaN returns true if either real(x) or imag(x) is NaN
// and neither is an infinity.
func IsNaN(x complex64) bool {
	switch {
	case math32.IsInf(real(x), 0) || math32.IsInf(imag(x), 0):
		return false
	case math32.IsNaN(real(x)) || math32.IsNaN(imag(x)):
		return true
	}
	return false
}

// NaN returns a complex ``not-a-number'' value.
func NaN() complex64 {
	nan := math32.NaN()
	return complex(nan, nan)
}
