package main

import (
	"bytes"
	"compress/gzip"
	"encoding/json"
	"net/http"
	"os"
	"time"

	"github.com/abrander/agento"
)

func main() {
	config := agento.Configuration{}
	err := config.LoadFromFile("/etc/agento.conf")

	if err != nil {
		agento.LogError("Configuration error: %s",
			err.Error())
		os.Exit(1)
	}

	agento.LogInfo("agento client started, reporting to %s", config.Client.ServerUrl)

	// We need to gather one unreported set of metrics. It's needed for
	// calculating deltas on first real report
	_ = agento.Gather()

	c := time.Tick(time.Second * time.Duration(config.Client.Interval))
	for _ = range c {
		stats := agento.Gather()
		json, err := json.Marshal(stats)
		if err != nil {
			agento.LogError(err.Error())
			continue
		}
		gz, err := gzip.NewReader(bytes.NewReader(json))
		if err != nil {
			agento.LogError(err.Error())
			continue
		}

		res, err := http.Post(config.Client.ServerUrl, "image/jpeg", gz)
		if err != nil {
			agento.LogError(err.Error())
			continue
		}
		res.Body.Close()

	}

	return
}
