// Copyright 2012, Google Inc. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package mysqlctl

import (
	"github.com/youtube/vitess/go/vt/mysqlctl/proto"
)

// GetPermissions lists the permissions on the mysqld
func GetPermissions(mysqld MysqlDaemon) (*proto.Permissions, error) {
	permissions := &proto.Permissions{}

	// get Users
	qr, err := mysqld.FetchSuperQuery("SELECT * FROM mysql.user")
	if err != nil {
		return nil, err
	}
	for _, row := range qr.Rows {
		permissions.UserPermissions = append(permissions.UserPermissions, proto.NewUserPermission(qr.Fields, row))
	}

	// get Dbs
	qr, err = mysqld.FetchSuperQuery("SELECT * FROM mysql.db")
	if err != nil {
		return nil, err
	}
	for _, row := range qr.Rows {
		permissions.DbPermissions = append(permissions.DbPermissions, proto.NewDbPermission(qr.Fields, row))
	}

	// get Hosts
	qr, err = mysqld.FetchSuperQuery("SELECT * FROM mysql.host")
	if err != nil {
		return nil, err
	}
	for _, row := range qr.Rows {
		permissions.HostPermissions = append(permissions.HostPermissions, proto.NewHostPermission(qr.Fields, row))
	}

	return permissions, nil
}
